@@ -28,7 +28,7 @@ class MigrationQuestioner:
             return True
         # Otherwise, we look to see if it has a migrations module
         # without any Python files in it, apart from __init__.py.
-        # Apps from the new app template will have these; the python
+        # Apps from the new app template will have these; the Python
         # file check will ensure we skip South ones.
         try:
             app_config = apps.get_app_config(app_label)
@@ -86,7 +86,7 @@ class InteractiveMigrationQuestioner(MigrationQuestioner):
         result = input("%s " % question)
         if not result and default is not None:
             return default
-        while len(result) < 1 or result[0].lower() not in "yn":
+        while not result or result[0].lower() not in "yn":
             result = input("Please answer yes or no: ")
         return result[0].lower() == "y"
 
