@@ -30,36 +30,6 @@ from django.db.models.fields.related import (  # isort:skip
 )
 
 
-def permalink(func):
-    """
-    Decorator that calls urls.reverse() to return a URL using parameters
-    returned by the decorated function "func".
-
-    "func" should be a function that returns a tuple in one of the
-    following formats:
-        (viewname, viewargs)
-        (viewname, viewargs, viewkwargs)
-    """
-    import warnings
-    from functools import wraps
-
-    from django.urls import reverse
-    from django.utils.deprecation import RemovedInDjango21Warning
-
-    warnings.warn(
-        'permalink() is deprecated in favor of calling django.urls.reverse() '
-        'in the decorated method.',
-        RemovedInDjango21Warning,
-        stacklevel=2,
-    )
-
-    @wraps(func)
-    def inner(*args, **kwargs):
-        bits = func(*args, **kwargs)
-        return reverse(bits[0], None, *bits[1:3])
-    return inner
-
-
 __all__ = aggregates_all + fields_all + indexes_all
 __all__ += [
     'ObjectDoesNotExist', 'signals',
@@ -72,5 +42,5 @@ __all__ += [
     'Prefetch', 'Q', 'QuerySet', 'prefetch_related_objects', 'DEFERRED', 'Model',
     'FilteredRelation',
     'ForeignKey', 'ForeignObject', 'OneToOneField', 'ManyToManyField',
-    'ManyToOneRel', 'ManyToManyRel', 'OneToOneRel', 'permalink',
+    'ManyToOneRel', 'ManyToManyRel', 'OneToOneRel',
 ]
