@@ -15,7 +15,8 @@ from django.core.exceptions import AppRegistryNotReady
 from django.core.signals import setting_changed
 from django.dispatch import receiver
 from django.utils.safestring import SafeData, mark_safe
-from django.utils.translation import LANGUAGE_SESSION_KEY
+
+from . import LANGUAGE_SESSION_KEY, to_language, to_locale
 
 # Translations are cached in a dictionary for every language.
 # The active translations are stored by threadid to make them thread local.
@@ -56,32 +57,6 @@ def reset_cache(**kwargs):
         get_supported_language_variant.cache_clear()
 
 
-def to_locale(language):
-    """Turn a language name (en-us) into a locale name (en_US)."""
-    language = language.lower()
-    parts = language.split('-')
-    try:
-        country = parts[1]
-    except IndexError:
-        return language
-    else:
-        # A language with > 2 characters after the dash only has its first
-        # character after the dash capitalized; e.g. sr-latn becomes sr_Latn.
-        # A language with 2 characters after the dash has both characters
-        # capitalized; e.g. en-us becomes en_US.
-        parts[1] = country.title() if len(country) > 2 else country.upper()
-    return parts[0] + '_' + '-'.join(parts[1:])
-
-
-def to_language(locale):
-    """Turn a locale name (en_US) into a language name (en-us)."""
-    p = locale.find('_')
-    if p >= 0:
-        return locale[:p].lower() + '-' + locale[p + 1:].lower()
-    else:
-        return locale.lower()
-
-
 class DjangoTranslation(gettext_module.GNUTranslations):
     """
     Set up the GNUTranslations context with regard to output charset.
@@ -146,7 +121,8 @@ class DjangoTranslation(gettext_module.GNUTranslations):
             localedir=localedir,
             languages=[self.__locale],
             codeset='utf-8',
-            fallback=use_null_fallback)
+            fallback=use_null_fallback,
+        )
 
     def _init_translation_catalog(self):
         """Create a base catalog using global django translations."""
@@ -202,6 +178,8 @@ class DjangoTranslation(gettext_module.GNUTranslations):
             self._catalog = other._catalog.copy()
         else:
             self._catalog.update(other._catalog)
+        if other._fallback:
+            self.add_fallback(other._fallback)
 
     def language(self):
         """Return the translation language."""
@@ -304,15 +282,15 @@ def gettext(message):
 
     eol_message = message.replace('\r\n', '\n').replace('\r', '\n')
 
-    if len(eol_message) == 0:
-        # Return an empty value of the corresponding type if an empty message
-        # is given, instead of metadata, which is the default gettext behavior.
-        result = type(message)("")
-    else:
+    if eol_message:
         _default = _default or translation(settings.LANGUAGE_CODE)
         translation_object = getattr(_active, "value", _default)
 
         result = translation_object.gettext(eol_message)
+    else:
+        # Return an empty value of the corresponding type if an empty message
+        # is given, instead of metadata, which is the default gettext behavior.
+        result = type(message)('')
 
     if isinstance(message, SafeData):
         return mark_safe(result)
@@ -377,7 +355,12 @@ def all_locale_paths():
     """
     globalpath = os.path.join(
         os.path.dirname(sys.modules[settings.__module__].__file__), 'locale')
-    return [globalpath] + list(settings.LOCALE_PATHS)
+    app_paths = []
+    for app_config in apps.get_app_configs():
+        locale_path = os.path.join(app_config.path, 'locale')
+        if os.path.exists(locale_path):
+            app_paths.append(locale_path)
+    return [globalpath] + list(settings.LOCALE_PATHS) + app_paths
 
 
 @functools.lru_cache(maxsize=1000)
@@ -394,10 +377,10 @@ def check_for_language(lang_code):
     # First, a quick check to make sure lang_code is well-formed (#21458)
     if lang_code is None or not language_code_re.search(lang_code):
         return False
-    for path in all_locale_paths():
-        if gettext_module.find('django', path, [to_locale(lang_code)]) is not None:
-            return True
-    return False
+    return any(
+        gettext_module.find('django', path, [to_locale(lang_code)]) is not None
+        for path in all_locale_paths()
+    )
 
 
 @functools.lru_cache()
@@ -411,11 +394,11 @@ def get_languages():
 @functools.lru_cache(maxsize=1000)
 def get_supported_language_variant(lang_code, strict=False):
     """
-    Return the language-code that's listed in supported languages, possibly
+    Return the language code that's listed in supported languages, possibly
     selecting a more generic variant. Raise LookupError if nothing is found.
 
-    If `strict` is False (the default), look for an alternative
-    country-specific variant when the currently checked is not found.
+    If `strict` is False (the default), look for a country-specific variant
+    when neither the language code nor its generic variant is found.
 
     lru_cache should have a maxsize to prevent from memory exhaustion attacks,
     as the provided language codes are taken from the HTTP request. See also
@@ -445,11 +428,10 @@ def get_supported_language_variant(lang_code, strict=False):
 
 def get_language_from_path(path, strict=False):
     """
-    Return the language-code if there is a valid language-code
-    found in the `path`.
+    Return the language code if there's a valid language code found in `path`.
 
-    If `strict` is False (the default), the function will look for an alternative
-    country-specific variant when the currently checked is not found.
+    If `strict` is False (the default), look for a country-specific variant
+    when neither the language code nor its generic variant is found.
     """
     regex_match = language_code_prefix_re.match(path)
     if not regex_match:
@@ -509,25 +491,26 @@ def get_language_from_request(request, check_path=False):
         return settings.LANGUAGE_CODE
 
 
+@functools.lru_cache(maxsize=1000)
 def parse_accept_lang_header(lang_string):
     """
     Parse the lang_string, which is the body of an HTTP Accept-Language
-    header, and return a list of (lang, q-value), ordered by 'q' values.
+    header, and return a tuple of (lang, q-value), ordered by 'q' values.
 
-    Return an empty list if there are any format errors in lang_string.
+    Return an empty tuple if there are any format errors in lang_string.
     """
     result = []
     pieces = accept_language_re.split(lang_string.lower())
     if pieces[-1]:
-        return []
+        return ()
     for i in range(0, len(pieces) - 1, 3):
         first, lang, priority = pieces[i:i + 3]
         if first:
-            return []
+            return ()
         if priority:
             priority = float(priority)
         else:
             priority = 1.0
         result.append((lang, priority))
     result.sort(key=lambda k: k[1], reverse=True)
-    return result
+    return tuple(result)
